@@ -14,11 +14,12 @@ using System.Windows.Forms;
 using System.ComponentModel;
 using NUnit.Framework;
 using CategoryAttribute = NUnit.Framework.CategoryAttribute;
+using System.Threading;
 
 namespace MonoTests.System.Windows.Forms
 {
 	[TestFixture]
-	public class MdiFormTest
+	public class MdiFormTest : TestHelper
 	{
 		private Form main;
 		private Form child1;
@@ -26,7 +27,7 @@ namespace MonoTests.System.Windows.Forms
 		private Form child3;
 
 		[TearDown]
-		public void TearDown ()
+		protected override void TearDown ()
 		{
 			if (main != null)
 				main.Dispose ();
@@ -40,11 +41,12 @@ namespace MonoTests.System.Windows.Forms
 			child1 = null;
 			child2 = null;
 			child3 = null;
+			base.TearDown ();
 		}
 	
 		// No attribute here since this is supposed to be called from 
 		// each test directly, not by nunit.
-		public void SetUp (bool only_create, bool only_text)
+		void SetUp (bool only_create, bool only_text)
 		{
 			SetUp (only_create, only_text, false);
 		}	
@@ -80,6 +82,96 @@ namespace MonoTests.System.Windows.Forms
 				child3.MdiParent = main;
 			}
 		}
+
+		[Test]
+		public void DeactivatedEventTest ()
+		{
+			using (Form f = new Form ()) {
+				Form c1 = new Form ();
+				Form c2 = new Form ();
+				EventLogger log1 = new EventLogger (c1);
+				EventLogger log2 = new EventLogger (c2);
+
+				f.IsMdiContainer = true;
+				f.ShowInTaskbar = false;
+				c2.MdiParent = c1.MdiParent = f;
+
+				c1.Show ();
+				c2.Show ();
+				Assert.AreEqual (0, log1.CountEvents ("Deactivate"), "#A1");
+				Assert.AreEqual (0, log2.CountEvents ("Deactivate"), "#A2");
+
+				f.Show ();
+				Assert.AreEqual (1, log1.CountEvents ("Deactivate"), "#B1");
+				Assert.AreEqual (0, log2.CountEvents ("Deactivate"), "#B2");
+
+				c1.Show ();
+				c2.Show ();
+				Assert.AreEqual (1, log1.CountEvents ("Deactivate"), "#C1");
+				Assert.AreEqual (0, log2.CountEvents ("Deactivate"), "#C2");
+
+				c1.Activate ();
+				Assert.AreEqual (1, log1.CountEvents ("Deactivate"), "#D1");
+				Assert.AreEqual (1, log2.CountEvents ("Deactivate"), "#D2");
+
+				c1.Activate ();
+				Assert.AreEqual (1, log1.CountEvents ("Deactivate"), "#E1");
+				Assert.AreEqual (1, log2.CountEvents ("Deactivate"), "#E2");
+
+				c2.Activate ();
+				Assert.AreEqual (2, log1.CountEvents ("Deactivate"), "#F1");
+				Assert.AreEqual (1, log2.CountEvents ("Deactivate"), "#F2");
+
+				c1.Activate ();
+				Assert.AreEqual (2, log1.CountEvents ("Deactivate"), "#G1");
+				Assert.AreEqual (2, log2.CountEvents ("Deactivate"), "#G2");
+			}
+		}
+		
+		[Test]
+		public void ActivatedEventTest ()
+		{
+			using (Form f = new Form ()) {
+				Form c1 = new Form ();
+				Form c2 = new Form ();
+				EventLogger log1 = new EventLogger (c1);
+				EventLogger log2 = new EventLogger (c2);
+
+				f.IsMdiContainer = true;
+				f.ShowInTaskbar = false;
+				c2.MdiParent = c1.MdiParent = f;
+
+				c1.Show ();
+				c2.Show ();
+				Assert.AreEqual (0, log1.CountEvents ("Activated"), "#A1");
+				Assert.AreEqual (0, log2.CountEvents ("Activated"), "#A2");
+				
+				f.Show ();
+				Assert.AreEqual (1, log1.CountEvents ("Activated"), "#B1");
+				Assert.AreEqual (1, log2.CountEvents ("Activated"), "#B2");
+				
+				c1.Show ();
+				c2.Show ();
+				Assert.AreEqual (1, log1.CountEvents ("Activated"), "#C1");
+				Assert.AreEqual (1, log2.CountEvents ("Activated"), "#C2");
+
+				c1.Activate ();
+				Assert.AreEqual (2, log1.CountEvents ("Activated"), "#D1");
+				Assert.AreEqual (1, log2.CountEvents ("Activated"), "#D2");
+
+				c1.Activate ();
+				Assert.AreEqual (2, log1.CountEvents ("Activated"), "#E1");
+				Assert.AreEqual (1, log2.CountEvents ("Activated"), "#E2");
+
+				c2.Activate ();
+				Assert.AreEqual (2, log1.CountEvents ("Activated"), "#F1");
+				Assert.AreEqual (2, log2.CountEvents ("Activated"), "#F2");
+
+				c1.Activate ();
+				Assert.AreEqual (3, log1.CountEvents ("Activated"), "#G1");
+				Assert.AreEqual (2, log2.CountEvents ("Activated"), "#G2");
+			}
+		}
 		
 		[Test]
 		public void ActiveControlTest ()
@@ -252,6 +344,68 @@ namespace MonoTests.System.Windows.Forms
 			TearDown ();
 		}
 		
+		[Test]
+		public void NewFormShouldAlsoBeMaximized ()
+		{
+			Form parent = new Form ();
+			
+			Form child1 = new Form ();
+			Form child2 = new Form ();
+			
+			parent.ShowInTaskbar = false;
+			parent.IsMdiContainer = true;
+			parent.ClientSize = new Size (300, 300);
+			
+			parent.Show ();
+			
+			child1.MdiParent = parent;
+			child1.Show ();
+			
+			child1.WindowState = FormWindowState.Maximized;
+			
+			child2.MdiParent = parent;
+			child2.ClientSize = new Size (100, 100);
+			child2.Show ();
+
+			Assert.IsTrue (child2.Width > 200, "Width should be maximized");
+			Assert.IsTrue (child2.Height > 200, "Height should be maximized");
+			
+			parent.Dispose ();
+		}
+
+#if NET_2_0
+		[Test]
+		public void RestoreWithMainMenuStrip ()
+		{
+			Form parent = new Form ();
+
+			Form child1 = new Form ();
+
+			parent.ShowInTaskbar = false;
+			parent.IsMdiContainer = true;
+			parent.ClientSize = new Size (500, 500);
+
+			MenuStrip ms = new MenuStrip ();
+			ms.Items.Add (new ToolStripMenuItem ("main", null, new ToolStripMenuItem ("item1")));
+			parent.Controls.Add (ms);
+			parent.MainMenuStrip = ms;
+
+			parent.Show ();
+
+			child1.MdiParent = parent;
+			child1.Show ();
+
+			Size s = child1.Size;
+			
+			child1.WindowState = FormWindowState.Maximized;
+			child1.WindowState = FormWindowState.Normal;
+
+			Assert.AreEqual (s, child1.Size, "A1");
+
+			parent.Dispose ();
+		}
+#endif
+
 		/* These tests are all failing on WinXP with XP Theme. 
 		 * offset seems to be 22,22 normally, and 22,29 with XP theme.
 		 * Find a way to test this reliably.
@@ -918,6 +1072,8 @@ namespace MonoTests.System.Windows.Forms
 			child3.Text = string.Empty;
 			Assert.AreEqual (string.Empty, main.Text, "#11");
 
+			main.Close ();
+			
 			child3.Dispose ();
 			child2.Dispose ();
 			child1.Dispose ();
@@ -1347,5 +1503,112 @@ namespace MonoTests.System.Windows.Forms
 			main.Show ();
 			Assert.AreEqual ("{Width=224, Height=37}", child1.ClientSize.ToString (), "#03");
 		}
+		
+		private string events_fired;
+		
+		[Test]
+		public void MdiClosingClosedEvent ()
+		{
+			Form f = new Form ();
+			f.IsMdiContainer = true;
+			
+			MdiChild mdi = new MdiChild ();
+			mdi.can_close = false;
+			mdi.MdiParent = f;
+
+			f.Closing += new CancelEventHandler (f_Closing);
+			f.Closed += new EventHandler (f_Closed);
+			mdi.Closing += new CancelEventHandler (mdi_Closing);
+			mdi.Closed += new EventHandler (mdi_Closed);
+
+#if NET_2_0
+			f.FormClosing += new FormClosingEventHandler (f_FormClosing);
+			f.FormClosed += new FormClosedEventHandler (f_FormClosed);
+			mdi.FormClosing += new FormClosingEventHandler (mdi_FormClosing);
+			mdi.FormClosed += new FormClosedEventHandler (mdi_FormClosed);
+#endif
+
+			f.Show ();
+			mdi.Show ();
+			
+			events_fired = string.Empty;
+			
+			f.Close ();
+			
+#if NET_2_0
+			Assert.AreEqual ("Child.Closing: True, Child.FormClosing: True, Parent.Closing: True, Parent.FormClosing: True, ", events_fired, "A1-2.0");
+#else
+			Assert.AreEqual ("Child.Closing: True, Parent.Closing: True, ", events_fired, "A1-1.1");
+#endif
+
+			events_fired = string.Empty;
+			mdi.can_close = true;
+			f.Close ();
+			
+#if NET_2_0
+			Assert.AreEqual ("Child.Closing: False, Child.FormClosing: False, Parent.Closing: False, Parent.FormClosing: False, Child.Closed, Child.FormClosed, Parent.Closed, Parent.FormClosed, ", events_fired, "A2-2.0");
+#else
+			Assert.AreEqual ("Child.Closing: False, Parent.Closing: False, Child.Closed, Parent.Closed, ", events_fired, "A2-1.1");
+#endif
+
+			f.Dispose ();
+		}
+
+#if NET_2_0
+		void mdi_FormClosed (object sender, FormClosedEventArgs e)
+		{
+			events_fired += "Child.FormClosed, ";
+		}
+
+		void mdi_FormClosing (object sender, FormClosingEventArgs e)
+		{
+			events_fired += string.Format ("Child.FormClosing: {0}, ", e.Cancel);
+		}
+
+		void f_FormClosed (object sender, FormClosedEventArgs e)
+		{
+			events_fired += "Parent.FormClosed, ";
+		}
+
+		void f_FormClosing (object sender, FormClosingEventArgs e)
+		{
+			events_fired += string.Format ("Parent.FormClosing: {0}, ", e.Cancel);
+		}
+#endif
+
+		void mdi_Closed (object sender, EventArgs e)
+		{
+			events_fired += "Child.Closed, ";
+		}
+
+		void mdi_Closing (object sender, CancelEventArgs e)
+		{
+			events_fired += string.Format ("Child.Closing: {0}, ", e.Cancel);
+		}
+
+		void f_Closed (object sender, EventArgs e)
+		{
+			events_fired += "Parent.Closed, ";
+		}
+
+		void f_Closing (object sender, CancelEventArgs e)
+		{
+			events_fired += string.Format ("Parent.Closing: {0}, ", e.Cancel);
+		}
+		
+		private class MdiChild : Form
+		{
+			public bool can_close = false;
+			
+			public MdiChild ()
+			{
+				Closing += new CancelEventHandler (MdiChild_Closing);
+			}
+
+			void MdiChild_Closing (object sender, CancelEventArgs e)
+			{
+				e.Cancel = !can_close;
+			}
+		}
 	}
 }
