@@ -237,11 +237,11 @@ namespace MonoTests.Microsoft.Build.BuildEngine.Various {
 			CheckItems (proj, "Item7", "A6", "A", "B", "C");
 		}
 
+		// The expression "@(Item1, '@(A,'')')" cannot be used in this context. 
+		// Item lists cannot be concatenated with other strings where an item list is expected. 
+		// Use a semicolon to separate multiple item lists.
 		[Test]
-		[ExpectedException (typeof (InvalidProjectFileException),
-			"The expression \"@(Item1, '@(A,'')')\" cannot be used in this context. " +
-			"Item lists cannot be concatenated with other strings where an item list is expected. " +
-			"Use a semicolon to separate multiple item lists.  ")]
+		[ExpectedException (typeof (InvalidProjectFileException))]
 		[Category ("NotWorking")]
 		public void TestItems7 ()
 		{
@@ -260,11 +260,11 @@ namespace MonoTests.Microsoft.Build.BuildEngine.Various {
 			proj.LoadXml (documentString);
 		}
 
+		// The expression "@(Item1, '@(A->'')')" cannot be used in this context.
+		// Item lists cannot be concatenated with other strings where an item list is expected.
+		// Use a semicolon to separate multiple item lists.
 		[Test]
-		[ExpectedException (typeof (InvalidProjectFileException),
-			"The expression \"@(Item1, '@(A->'')')\" cannot be used in this context. " +
-			"Item lists cannot be concatenated with other strings where an item list is expected. " +
-			"Use a semicolon to separate multiple item lists.  ")]
+		[ExpectedException (typeof (InvalidProjectFileException))]
 		[Category ("NotWorking")]
 		public void TestItems8 ()
 		{
@@ -283,11 +283,11 @@ namespace MonoTests.Microsoft.Build.BuildEngine.Various {
 			proj.LoadXml (documentString);
 		}
 
+		// The expression "@(Item1, '@(A->'','')')" cannot be used in this context.
+		// Item lists cannot be concatenated with other strings where an item list is expected.
+		// Use a semicolon to separate multiple item lists.
 		[Test]
-		[ExpectedException (typeof (InvalidProjectFileException),
-			"The expression \"@(Item1, '@(A->'','')')\" cannot be used in this context. " +
-			"Item lists cannot be concatenated with other strings where an item list is expected. " +
-			"Use a semicolon to separate multiple item lists.  ")]
+		[ExpectedException (typeof (InvalidProjectFileException))]
 		[Category ("NotWorking")]
 		public void TestItems9 ()
 		{
@@ -459,6 +459,7 @@ namespace MonoTests.Microsoft.Build.BuildEngine.Various {
 						"  $(C)   Foo    $(C)  Bar ; $(B)   ",
 						"@(A);$(C)",
 						"@(A);A;B;C",
+						"  abc;  @(A)  ;  $(C)  ;foo",
 					}) + "</Project>";
 
 
@@ -473,6 +474,7 @@ namespace MonoTests.Microsoft.Build.BuildEngine.Various {
 			CheckItems (proj, "I5", "A5", "A", "Foo    A", "Bar", "A", "B");
 			CheckItems (proj, "I6", "A6", "A", "B", "C", "A");
 			CheckItems (proj, "I7", "A7", "A", "B", "C", "A", "B", "C");
+			CheckItems (proj, "I8", "A8", "abc", "A", "B", "C", "A", "foo");
 		}
 
 		[Test]
@@ -804,5 +806,79 @@ namespace MonoTests.Microsoft.Build.BuildEngine.Various {
 			Assert.IsFalse (proj.Build ("1"));
 		}
 
+		[Test]
+		public void TestItemsInTarget8 ()
+		{
+			Engine engine = new Engine (Consts.BinPath);
+			Project proj = engine.CreateNewProject ();
+
+			string documentString = @"
+				<Project xmlns=""http://schemas.microsoft.com/developer/msbuild/2003"">
+					<PropertyGroup>
+						<Foo>Five</Foo>
+					</PropertyGroup>
+					<ItemGroup>
+						<A Include='A'>
+							<M>True</M>
+							<M>False</M>
+						</A>
+					</ItemGroup>
+				</Project>
+			";
+
+			proj.LoadXml (documentString);
+
+			Assert.AreEqual (1, proj.EvaluatedItems.Count, "A1");
+			BuildItem bi = proj.EvaluatedItems [0];
+			Assert.AreEqual ("False", bi.GetMetadata ("M"), "A2");
+		}
+
+
+		[Test]
+		public void TestItemsInTarget9 ()
+		{
+			Engine engine = new Engine (Consts.BinPath);
+			Project proj = engine.CreateNewProject ();
+
+			string documentString = @"
+				<Project xmlns=""http://schemas.microsoft.com/developer/msbuild/2003"">
+					<PropertyGroup>
+						<Foo>Five</Foo>
+					</PropertyGroup>
+					<ItemGroup>
+						<A Include='A'>
+							<M Condition="" '$(Foo)' == 'Five' "">True</M>
+							<M Condition="" '$(Foo)' != 'Five' "">False</M>
+						</A>
+					</ItemGroup>
+				</Project>
+			";
+
+			proj.LoadXml (documentString);
+
+			Assert.AreEqual (1, proj.EvaluatedItems.Count, "A1");
+			BuildItem bi = proj.EvaluatedItems [0];
+			Assert.AreEqual ("True", bi.GetMetadata ("M"), "A2");
+			Assert.AreEqual (0, bi.Condition.Length, "A3");
+
+			BuildItemGroup big = proj.GetEvaluatedItemsByNameIgnoringCondition ("A");
+			Assert.AreEqual (1, big.Count, "A4");
+			bi = big [0];
+			Assert.AreEqual ("True", bi.GetMetadata ("M"), "A5");
+			Assert.AreEqual ("True", bi.GetEvaluatedMetadata ("M"), "A6");
+
+			/*proj.SetProperty ("Foo", "Six");
+			proj.Build ();
+			bi = proj.GetEvaluatedItemsByName ("A") [0];
+			Assert.AreEqual ("False", bi.GetMetadata ("M"), "A7");
+			Assert.AreEqual ("False", bi.GetEvaluatedMetadata ("M"), "A7a");
+			Assert.AreEqual (0, bi.Condition.Length, "A8");
+
+			big = proj.GetEvaluatedItemsByNameIgnoringCondition ("A");
+			Assert.AreEqual (1, big.Count, "A9");
+			bi = big [0];
+			Assert.AreEqual ("True", bi.GetMetadata ("M"), "A10");
+			Assert.AreEqual ("True", bi.GetEvaluatedMetadata ("M"), "A11");*/
+		}
 	}
 }
