@@ -563,11 +563,6 @@ namespace Mono.Xml
 			}
 		}
 
-		private XmlException NotWFError (string message)
-		{
-			return new XmlException (this as IXmlLineInfo, BaseURI, message);
-		}
-
 		private void HandleError (string message, XmlSeverityType severity)
 		{
 			if (validatingReader != null &&
@@ -973,7 +968,12 @@ namespace Mono.Xml
 			}
 		}
 
-		public override bool ReadAttributeValue ()
+#if NET_2_1
+		internal
+#else
+		public
+#endif
+		override bool ReadAttributeValue ()
 		{
 			if (consumedAttribute)
 				return false;
@@ -990,7 +990,12 @@ namespace Mono.Xml
 				return reader.ReadAttributeValue ();
 		}
 
-		public override void ResolveEntity ()
+#if NET_2_1
+		internal
+#else
+		public
+#endif
+		override void ResolveEntity ()
 		{
 			reader.ResolveEntity ();
 		}
@@ -1028,7 +1033,12 @@ namespace Mono.Xml
 			get { return reader.EOF; }
 		}
 
-		public override bool HasValue {
+#if NET_2_1
+		internal
+#else
+		public
+#endif
+		override bool HasValue {
 			get {
 				return currentAttribute >= 0 ? true :
 					currentTextValue != null ? true :
