@@ -4,9 +4,12 @@
 // Authors:
 //   Gonzalo Paniagua Javier (gonzalo@ximian.com)
 //   Andreas Nahr (ClassDevelopment@A-SoftTech.com)
+//   Ivan N. Zlatev (contact@i-nz.net)
+// 
 //
 // (C) 2002 Ximian, Inc (http://www.ximian.com)
 // (C) 2003 Andreas Nahr
+// (C) 2008 Novell, Inc (http://www.novell.com)
 //
 
 //
@@ -42,21 +45,94 @@ namespace System.ComponentModel
 public sealed class TypeDescriptor
 {
 	private static readonly object creatingDefaultConverters = new object ();
-	private static Hashtable defaultConverters;
+	private static ArrayList defaultConverters;
 	private static IComNativeDescriptorHandler descriptorHandler;
 	private static Hashtable componentTable = new Hashtable ();
 	private static Hashtable typeTable = new Hashtable ();
+	private static Hashtable editors;
 
 	private TypeDescriptor ()
 	{
 	}
 
-	[MonoTODO]
-	public static void AddEditorTable (Type editorBaseType, Hashtable table)
+#if NET_2_0
+	[MonoNotSupported ("")]
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+	public static Type ComObjectType {
+		get { throw new NotImplementedException (); }
+	}
+
+	[MonoNotSupported("")]
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+	public static TypeDescriptionProvider AddAttributes (object instance, params Attribute [] attributes)
 	{
 		throw new NotImplementedException ();
 	}
 
+	[MonoNotSupported("")]
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+	public static TypeDescriptionProvider AddAttributes (Type type, params Attribute [] attributes)
+	{
+		throw new NotImplementedException ();
+	}
+
+	[MonoNotSupported("")]
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+	public static void AddProvider (TypeDescriptionProvider provider, object instance)
+	{
+		throw new NotImplementedException ();
+	}
+
+	[MonoNotSupported("")]
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+	public static void AddProvider (TypeDescriptionProvider provider, Type type)
+	{
+		throw new NotImplementedException ();
+	}
+
+	[MonoTODO]
+	public static object CreateInstance (IServiceProvider provider, Type objectType, Type [] argTypes, object [] args)
+	{
+		if (objectType == null)
+			throw new ArgumentNullException ("objectType");
+
+		object instance = null;
+
+		if (provider != null) {
+			TypeDescriptionProvider typeDescrProvider = provider.GetService (typeof (TypeDescriptionProvider)) as TypeDescriptionProvider;
+			if (typeDescrProvider != null)
+				instance = typeDescrProvider.CreateInstance (provider, objectType, argTypes, args);
+		}
+
+		// TODO: also search and use the internal providers table once Add/RemoveProvider have been implemented
+
+		if (instance == null)
+			instance = Activator.CreateInstance (objectType, args);
+
+		return instance;
+	}
+#endif
+
+#if NET_2_0
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+#endif
+#if !NET_2_1
+	public
+#else
+	internal
+#endif
+	static void AddEditorTable (Type editorBaseType, Hashtable table)
+	{
+		if (editorBaseType == null)
+			throw new ArgumentNullException ("editorBaseType");
+
+		if (editors == null)
+			editors = new Hashtable ();
+
+		if (!editors.ContainsKey (editorBaseType))
+			editors [editorBaseType] = table;
+	}
+
 	public static IDesigner CreateDesigner(IComponent component, Type designerBaseType)
 	{
 		string tn = designerBaseType.AssemblyQualifiedName;
@@ -65,7 +141,9 @@ public sealed class TypeDescriptor
 		foreach (Attribute at in col) {
 			DesignerAttribute dat = at as DesignerAttribute;
 			if (dat != null && tn == dat.DesignerBaseTypeName) {
-				return (IDesigner) Activator.CreateInstance (GetTypeFromName (component, dat.DesignerTypeName));
+				Type designerType = GetTypeFromName (component, dat.DesignerTypeName);
+				if (designerType != null)
+					return (IDesigner) Activator.CreateInstance (designerType);
 			}
 		}
 				
@@ -76,7 +154,7 @@ public sealed class TypeDescriptor
 	public static EventDescriptor CreateEvent (Type componentType,
 						   string name,
 						   Type type,
-						   Attribute [] attributes)
+						   params Attribute [] attributes)
 	{
 		return new ReflectionEventDescriptor (componentType, name, type, attributes);
 	}
@@ -84,7 +162,7 @@ public sealed class TypeDescriptor
 	[ReflectionPermission (SecurityAction.LinkDemand, TypeInformation = true, MemberAccess = true)]
 	public static EventDescriptor CreateEvent (Type componentType,
 						   EventDescriptor oldEventDescriptor,
-						   Attribute [] attributes)
+						   params Attribute [] attributes)
 	{
 		return new ReflectionEventDescriptor (componentType, oldEventDescriptor, attributes);
 	}
@@ -93,7 +171,7 @@ public sealed class TypeDescriptor
 	public static PropertyDescriptor CreateProperty (Type componentType,
 							 string name,
 							 Type type,
-							 Attribute [] attributes)
+							 params Attribute [] attributes)
 	{
 		return new ReflectionPropertyDescriptor (componentType, name, type, attributes);
 	}
@@ -101,7 +179,7 @@ public sealed class TypeDescriptor
 	[ReflectionPermission (SecurityAction.LinkDemand, TypeInformation = true, MemberAccess = true)]
 	public static PropertyDescriptor CreateProperty (Type componentType,
 							 PropertyDescriptor oldPropertyDescriptor,
-							 Attribute [] attributes)
+							 params Attribute [] attributes)
 	{
 		return new ReflectionPropertyDescriptor (componentType, oldPropertyDescriptor, attributes);
 	}
@@ -119,13 +197,16 @@ public sealed class TypeDescriptor
 		return GetAttributes (component, false);
 	}
 
+#if NET_2_0
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+#endif
 	public static AttributeCollection GetAttributes (object component, bool noCustomTypeDesc)
 	{
 		if (component == null)
-		    return AttributeCollection.Empty;
+			return AttributeCollection.Empty;
 
 		if (noCustomTypeDesc == false && component is ICustomTypeDescriptor) {
-		    return ((ICustomTypeDescriptor) component).GetAttributes ();
+			return ((ICustomTypeDescriptor) component).GetAttributes ();
 		} else {
 			IComponent com = component as IComponent;
 			if (com != null && com.Site != null)
@@ -140,20 +221,23 @@ public sealed class TypeDescriptor
 		return GetClassName (component, false);
 	}
 
+#if NET_2_0
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+#endif
 	public static string GetClassName (object component, bool noCustomTypeDesc)
 	{
 		if (component == null)
-		    throw new ArgumentNullException ("component", "component cannot be null");
+			throw new ArgumentNullException ("component", "component cannot be null");
 
 		if (noCustomTypeDesc == false && component is ICustomTypeDescriptor) {
-		    String res = ((ICustomTypeDescriptor) component).GetClassName ();
+			String res = ((ICustomTypeDescriptor) component).GetClassName ();
 			if (res == null)
 				res = ((ICustomTypeDescriptor) component).GetComponentName ();
 			if (res == null)
 				res = component.GetType ().FullName;
 			return res;
 		} else {
-		    return component.GetType ().FullName;
+			return component.GetType ().FullName;
 		}
 	}
 
@@ -162,13 +246,16 @@ public sealed class TypeDescriptor
 		return GetComponentName (component, false);
 	}
 
+#if NET_2_0
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+#endif
 	public static string GetComponentName (object component, bool noCustomTypeDesc)
 	{
 		if (component == null)
-		    throw new ArgumentNullException ("component", "component cannot be null");
+			throw new ArgumentNullException ("component", "component cannot be null");
 
 		if (noCustomTypeDesc == false && component is ICustomTypeDescriptor) {
-		    return ((ICustomTypeDescriptor) component).GetComponentName ();
+			return ((ICustomTypeDescriptor) component).GetComponentName ();
 		} else {
 			IComponent c = component as IComponent;
 			if (c != null && c.Site != null)
@@ -181,11 +268,28 @@ public sealed class TypeDescriptor
 		}
 	}
 
+#if NET_2_0
+	[MonoNotSupported("")]
+	public static string GetFullComponentName (object component)
+	{
+		throw new NotImplementedException ();
+	}
+
+	[MonoNotSupported("")]
+	public static string GetClassName (Type componentType)
+	{
+		throw new NotImplementedException ();
+	}
+#endif
+
 	public static TypeConverter GetConverter (object component)
 	{
 		return GetConverter (component, false);
 	}
 
+#if NET_2_0
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+#endif
 	public static TypeConverter GetConverter (object component, bool noCustomTypeDesc)
 	{
 		if (component == null)
@@ -195,55 +299,55 @@ public sealed class TypeDescriptor
 			return ((ICustomTypeDescriptor) component).GetConverter ();
 		} 
 		else {
-			Type t = null;
+			Type converterType = null;
 			AttributeCollection atts = GetAttributes (component, false);
 			TypeConverterAttribute tca = (TypeConverterAttribute) atts[typeof(TypeConverterAttribute)];
-			if (tca != null && tca.ConverterTypeName.Length > 0) {
-				t = GetTypeFromName (component as IComponent, tca.ConverterTypeName);
-			}
+			if (tca != null && tca.ConverterTypeName.Length > 0)
+				converterType = GetTypeFromName (component as IComponent, tca.ConverterTypeName);
+			if (converterType == null)
+				converterType = FindDefaultConverterType (component.GetType ());
 			
-			if (t != null) {
-				ConstructorInfo ci = t.GetConstructor (new Type[] { typeof(Type) });
+			if (converterType != null) {
+				ConstructorInfo ci = converterType.GetConstructor (new Type[] { typeof(Type) });
 				if (ci != null)
 					return (TypeConverter) ci.Invoke (new object[] { component.GetType () });
 				else
-					return (TypeConverter) Activator.CreateInstance (t);
-			}
-			else
-				return GetConverter (component.GetType ());
+					return (TypeConverter) Activator.CreateInstance (converterType);
+			} else
+				return null;
 		}
 	}
 
-	private static Hashtable DefaultConverters
+	private static ArrayList DefaultConverters
 	{
 		get {
 			lock (creatingDefaultConverters) {
 				if (defaultConverters != null)
 					return defaultConverters;
 				
-				defaultConverters = new Hashtable ();
-				defaultConverters.Add (typeof (bool), typeof (BooleanConverter));
-				defaultConverters.Add (typeof (byte), typeof (ByteConverter));
-				defaultConverters.Add (typeof (sbyte), typeof (SByteConverter));
-				defaultConverters.Add (typeof (string), typeof (StringConverter));
-				defaultConverters.Add (typeof (char), typeof (CharConverter));
-				defaultConverters.Add (typeof (short), typeof (Int16Converter));
-				defaultConverters.Add (typeof (int), typeof (Int32Converter));
-				defaultConverters.Add (typeof (long), typeof (Int64Converter));
-				defaultConverters.Add (typeof (ushort), typeof (UInt16Converter));
-				defaultConverters.Add (typeof (uint), typeof (UInt32Converter));
-				defaultConverters.Add (typeof (ulong), typeof (UInt64Converter));
-				defaultConverters.Add (typeof (float), typeof (SingleConverter));
-				defaultConverters.Add (typeof (double), typeof (DoubleConverter));
-				defaultConverters.Add (typeof (decimal), typeof (DecimalConverter));
-				defaultConverters.Add (typeof (object), typeof (TypeConverter));
-				defaultConverters.Add (typeof (void), typeof (TypeConverter));
-				defaultConverters.Add (typeof (Array), typeof (ArrayConverter));
-				defaultConverters.Add (typeof (CultureInfo), typeof (CultureInfoConverter));
-				defaultConverters.Add (typeof (DateTime), typeof (DateTimeConverter));
-				defaultConverters.Add (typeof (Guid), typeof (GuidConverter));
-				defaultConverters.Add (typeof (TimeSpan), typeof (TimeSpanConverter));
-				defaultConverters.Add (typeof (ICollection), typeof (CollectionConverter));
+				defaultConverters = new ArrayList ();
+				defaultConverters.Add (new DictionaryEntry (typeof (bool), typeof (BooleanConverter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (byte), typeof (ByteConverter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (sbyte), typeof (SByteConverter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (string), typeof (StringConverter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (char), typeof (CharConverter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (short), typeof (Int16Converter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (int), typeof (Int32Converter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (long), typeof (Int64Converter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (ushort), typeof (UInt16Converter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (uint), typeof (UInt32Converter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (ulong), typeof (UInt64Converter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (float), typeof (SingleConverter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (double), typeof (DoubleConverter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (decimal), typeof (DecimalConverter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (void), typeof (TypeConverter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (Array), typeof (ArrayConverter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (CultureInfo), typeof (CultureInfoConverter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (DateTime), typeof (DateTimeConverter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (Guid), typeof (GuidConverter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (TimeSpan), typeof (TimeSpanConverter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (ICollection), typeof (CollectionConverter)));
+				defaultConverters.Add (new DictionaryEntry (typeof (Enum), typeof (EnumConverter)));
 			}
 			return defaultConverters;
 		}
@@ -251,70 +355,66 @@ public sealed class TypeDescriptor
 	
 	public static TypeConverter GetConverter (Type type)
 	{
-		TypeConverterAttribute tca = null;
-		Type t = null;
-		object [] atts = type.GetCustomAttributes (typeof(TypeConverterAttribute), true);
+		if (type == null)
+			throw new ArgumentNullException ("type");
 		
-		if (atts.Length > 0)
-			tca = (TypeConverterAttribute)atts[0];
-		
-		if (tca != null) {
-			t = GetTypeFromName (null, tca.ConverterTypeName);
-		}
-		
-		if (t == null) {
-			if (type.IsEnum) {
-				// EnumConverter needs to know the enum type
-				return new EnumConverter(type);
-			} else if (type.IsArray) {
-				return new ArrayConverter ();
-			}
-		}
-		
-		if (t == null)
-			t = FindConverterType (type);
+		Type converterType = null;
+		AttributeCollection atts = GetAttributes (type);
+		TypeConverterAttribute tca = (TypeConverterAttribute) atts[typeof(TypeConverterAttribute)];
+		if (tca != null && tca.ConverterTypeName.Length > 0)
+			converterType = GetTypeFromName (null, tca.ConverterTypeName);
+		if (converterType == null)
+			converterType = FindDefaultConverterType (type);
 
-		if (t != null) {
-			Exception exc = null;
-			try {
-				return (TypeConverter) Activator.CreateInstance (t);
-			} catch (MissingMethodException e) {
-				exc = e;
-			}
-
-			try {
-				return (TypeConverter) Activator.CreateInstance (t, new object [] {type});
-			} catch (MissingMethodException e) {
-				throw exc;
-			}
+		if (converterType != null) {
+			ConstructorInfo ci = converterType.GetConstructor (new Type[] { typeof(Type) });
+			if (ci != null)
+				return (TypeConverter) ci.Invoke (new object[] { type });
+			else
+				return (TypeConverter) Activator.CreateInstance (converterType);
 		}
-
-		return new ReferenceConverter (type);    // Default?
-	}
-
-	private static Type FindConverterType (Type type)
-	{
-		Type t = null;
-		
-		// Is there a default converter
-		t = (Type) DefaultConverters [type];
-		if (t != null)
-			return t;
-		
-		// Find default converter with a type this type is assignable to
-		foreach (Type defType in DefaultConverters.Keys) {
-			if (defType.IsInterface && defType.IsAssignableFrom (type)) {
-				return (Type) DefaultConverters [defType];
-			}
-		}
-		
-		// Nothing found, try the same with our base type
-		if (type.BaseType != null)
-			return FindConverterType (type.BaseType);
 		else
 			return null;
 	}
 
+	private static Type FindDefaultConverterType (Type type)
+	{
+		Type converterType = null;
+		if (type != null) {
+#if NET_2_0
+			if (type.IsGenericType && type.GetGenericTypeDefinition() == typeof(Nullable<>))
+				return typeof(NullableConverter);
+#endif
+			// Is there a default converter
+			foreach (DictionaryEntry entry in DefaultConverters) {
+				if ((Type)entry.Key == type)
+					return (Type)entry.Value;
+			}
+		}
+		
+		// Find default converter with a baseType this baseType is assignable to
+		Type baseType = type;
+		while (baseType != null && baseType != typeof (object)) {
+			foreach (DictionaryEntry entry in DefaultConverters) {
+				Type defType = (Type)entry.Key;
+				if (defType.IsAssignableFrom (baseType)) {
+					converterType = (Type)entry.Value;
+					break;
+				}
+			}
+			baseType = baseType.BaseType;
+		}
+
+		if (converterType == null) {
+			if (type != null && type.IsInterface)
+				converterType = typeof (ReferenceConverter);
+			else
+				converterType = typeof (TypeConverter);
+		}
+
+		return converterType;
+	}
+
 	public static EventDescriptor GetDefaultEvent (Type componentType)
 	{
 		return GetTypeInfo (componentType).GetDefaultEvent ();
@@ -325,6 +425,9 @@ public sealed class TypeDescriptor
 		return GetDefaultEvent (component, false);
 	}
 
+#if NET_2_0
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+#endif
 	public static EventDescriptor GetDefaultEvent (object component, bool noCustomTypeDesc)
 	{
 		if (!noCustomTypeDesc && (component is ICustomTypeDescriptor))
@@ -348,6 +451,9 @@ public sealed class TypeDescriptor
 		return GetDefaultProperty (component, false);
 	}
 
+#if NET_2_0
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+#endif
 	public static PropertyDescriptor GetDefaultProperty (object component, bool noCustomTypeDesc)
 	{
 		if (!noCustomTypeDesc && (component is ICustomTypeDescriptor))
@@ -360,38 +466,92 @@ public sealed class TypeDescriptor
 				return GetTypeInfo (component.GetType()).GetDefaultProperty ();
 		}
 	}
+		
+	internal static object CreateEditor (Type t, Type componentType)
+	{
+		if (t == null) 
+			return null;
+
+		try {
+			return Activator.CreateInstance (t);
+		} catch {}
+
+		try {
+			return Activator.CreateInstance (t, new object [] {componentType});
+		} catch {}
+
+		return null;
+	}
+		
+	private static object FindEditorInTable (Type componentType, Type editorBaseType, Hashtable table)
+	{
+		object editorReference = null;
+		object editor = null;
+		
+		if (componentType == null || editorBaseType == null || table == null)
+			return null;
+			
+		Type ct = componentType;
+		while (ct != null) {						
+			editorReference = table [ct];
+			if (editorReference != null)
+				break;			
+			ct = ct.BaseType;
+		}
+		
+		if (editorReference == null) {
+			foreach (Type iface in componentType.GetInterfaces ()) {
+				editorReference = table [iface];
+				if (editorReference != null) 
+					break;
+			}
+		}
+				
+		if (editorReference == null)
+			return null;
+				
+		if (editorReference is string)
+			editor = CreateEditor (Type.GetType ((string) editorReference), componentType);
+		else if (editorReference is Type)
+			editor = CreateEditor ((Type) editorReference, componentType);
+		else if (editorReference.GetType ().IsSubclassOf (editorBaseType))
+			editor = editorReference;
+		
+		if (editor != null) 
+			table [componentType] = editor;
+		
+		return editor;
+	}
 
 	public static object GetEditor (Type componentType, Type editorBaseType)
 	{
-		Type t = null;
+		Type editorType = null;
+		object editor = null;
 		object [] atts = componentType.GetCustomAttributes (typeof(EditorAttribute), true);
-		if (atts == null || atts.Length == 0)
-			return null;
 		
-		
-		foreach (EditorAttribute ea in atts)
-		{
-			t = GetTypeFromName (null, ea.EditorTypeName);
-			if (t.IsSubclassOf(editorBaseType))
-				break;
-		}
-
-		if (t != null) {
-			Exception exc = null;
-			try {
-				return Activator.CreateInstance (t);
-			} catch (MissingMethodException e) {
-				exc = e;
-			}
-
-			try {
-				return Activator.CreateInstance (t, new object [] {componentType});
-			} catch (MissingMethodException e) {
-				throw exc;
+		if (atts != null && atts.Length != 0) {
+			foreach (EditorAttribute ea in atts) {
+				editorType = GetTypeFromName (null, ea.EditorTypeName);
+				if (editorType != null && editorType.IsSubclassOf(editorBaseType))
+					break;
 			}
 		}
+		
+		if (editorType != null)
+			editor = CreateEditor (editorType, componentType);
+			
+		if (editorType == null || editor == null) {
+#if !TARGET_JVM
+			// Make sure the editorBaseType's static constructor has been called,
+			// since that's where we're putting the initialization of its editor table.
+			
+			System.Runtime.CompilerServices.RuntimeHelpers.RunClassConstructor (editorBaseType.TypeHandle);
+#endif				
+			if (editors != null)
+				editor = FindEditorInTable (componentType, editorBaseType, editors [editorBaseType] as Hashtable);
+		}
 
-		return null;    // No editor specified
+		return editor;
 	}
 
 	public static object GetEditor (object component, Type editorBaseType)
@@ -399,6 +559,9 @@ public sealed class TypeDescriptor
 		return GetEditor (component, editorBaseType, false);
 	}
 
+#if NET_2_0
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+#endif
 	public static object GetEditor (object component, Type editorBaseType, bool noCustomTypeDesc)
 	{
 		if (component == null)
@@ -439,6 +602,9 @@ public sealed class TypeDescriptor
 		return GetEvents (component, attributes, false);
 	}
 
+#if NET_2_0
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+#endif
 	public static EventDescriptorCollection GetEvents (object component, bool noCustomTypeDesc)
 	{
 		if (!noCustomTypeDesc && (component is ICustomTypeDescriptor))
@@ -457,6 +623,9 @@ public sealed class TypeDescriptor
 		return GetTypeInfo (componentType).GetEvents (attributes);
 	}
 
+#if NET_2_0
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+#endif
 	public static EventDescriptorCollection GetEvents (object component, Attribute [] attributes, bool noCustomTypeDesc)
 	{
 		if (!noCustomTypeDesc && (component is ICustomTypeDescriptor))
@@ -501,6 +670,9 @@ public sealed class TypeDescriptor
 		}
 	}
 
+#if NET_2_0
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+#endif
 	public static PropertyDescriptorCollection GetProperties (object component, bool noCustomTypeDesc)
 	{
 		if (component == null)
@@ -522,6 +694,78 @@ public sealed class TypeDescriptor
 		return GetTypeInfo (componentType).GetProperties (attributes);
 	}
 
+#if NET_2_0
+	[MonoNotSupported ("")]
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+	public static TypeDescriptionProvider GetProvider (object instance)
+	{
+		throw new NotImplementedException ();
+	}
+
+	[MonoNotSupported ("")]
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+	public static TypeDescriptionProvider GetProvider (Type type)
+	{
+		throw new NotImplementedException ();
+	}
+
+	[MonoNotSupported ("")]
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+	public static Type GetReflectionType (object instance)
+	{
+		throw new NotImplementedException ();
+	}
+
+	[MonoNotSupported ("")]
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+	public static Type GetReflectionType (Type type)
+	{
+		throw new NotImplementedException ();
+	}
+
+	[MonoNotSupported("Associations not supported")]
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+	public static void CreateAssociation (object primary, object secondary)
+	{
+		throw new NotImplementedException ();
+	}
+
+	[MonoNotSupported ("Associations not supported")]
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+	public static object GetAssociation (Type type, object primary)
+	{
+		throw new NotImplementedException ();
+	}
+
+	[MonoNotSupported ("Associations not supported")]
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+	public static void RemoveAssociation (object primary, object secondary)
+	{
+		throw new NotImplementedException ();
+	}
+
+	[MonoNotSupported ("Associations not supported")]
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+	public static void RemoveAssociations (object primary)
+	{
+		throw new NotImplementedException ();
+	}
+
+	[MonoNotSupported ("")]
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+	public static void RemoveProvider (TypeDescriptionProvider provider, object instance)
+	{
+		throw new NotImplementedException ();
+	}
+
+	[MonoNotSupported ("")]
+	[EditorBrowsable (EditorBrowsableState.Advanced)]
+	public static void RemoveProvider (TypeDescriptionProvider provider, Type type)
+	{
+		throw new NotImplementedException ();
+	}
+#endif
+
 	public static void SortDescriptorArray (IList infos)
 	{
 		string[] names = new string [infos.Count];
@@ -536,6 +780,10 @@ public sealed class TypeDescriptor
 			infos.Add (ob);
 	}
 
+#if NET_2_0
+	// well, ComObjectType is not implemented, but we don't support COM anyways ...
+	[Obsolete ("Use ComObjectType")]
+#endif
 	public static IComNativeDescriptorHandler ComNativeDescriptorHandler {
 		[PermissionSet (SecurityAction.LinkDemand, Unrestricted = true)]
 		get { return descriptorHandler; }
@@ -614,17 +862,17 @@ public sealed class TypeDescriptor
 		}
 	}
 	
-	static Type GetTypeFromName (IComponent component, string typeName)
+	private static Type GetTypeFromName (IComponent component, string typeName)
 	{
+		Type type = null;
 		if (component != null && component.Site != null) {
 			ITypeResolutionService resver = (ITypeResolutionService) component.Site.GetService (typeof(ITypeResolutionService));
 			if (resver != null)
-				return resver.GetType (typeName, true, false);
+				type = resver.GetType (typeName);
 		}
-		
-		Type t = Type.GetType (typeName);
-		if (t == null) throw new ArgumentException ("Type '" + typeName + "' not found");
-		return t;
+		if (type == null)
+			type = Type.GetType (typeName);
+		return type;
 	}
 }
 
@@ -720,26 +968,43 @@ public sealed class TypeDescriptor
 				return _attributes;
 			
 			bool cache = true;
-			object[] ats = _infoType.GetCustomAttributes (true);
-			Hashtable t = new Hashtable ();
+			ArrayList attributesList = new ArrayList ();
 
-			// Filter the custom attributes, so that only the top
-			// level of the same type are left.
-			//
-			for (int i = ats.Length -1; i >= 0; i--) {
-				t [((Attribute) ats[i]).TypeId] = ats[i];
+			// 1) Attributes of the type
+			foreach (Attribute attribute in _infoType.GetCustomAttributes (false))
+				attributesList.Add (attribute);
+
+			// 2) Attributes of the base types
+			Type baseType = _infoType.BaseType;
+			while (baseType != null && baseType != typeof (object)) {
+				foreach (Attribute attribute in baseType.GetCustomAttributes (false))
+					attributesList.Add (attribute);
+				baseType = baseType.BaseType;
 			}
-					
+
+			// 3) Attributes of the type's implemented interfaces and their interfaces as well
+			foreach (Type inface in _infoType.GetInterfaces ())
+				foreach (Attribute attribute in TypeDescriptor.GetAttributes (inface))
+					attributesList.Add (attribute);
+
+			// Filter out duplicate attributes, so that the base types have higher precedence 
+			// than the interfaces and the type higher than both.
+			Hashtable attributesTable = new Hashtable ();
+			for (int i = attributesList.Count - 1; i >= 0; i--) {
+				Attribute attribute = (Attribute)attributesList[i];
+				attributesTable[attribute.TypeId] = attribute;
+			}
+
 			if (comp != null && comp.Site != null) 
 			{
 				ITypeDescriptorFilterService filter = (ITypeDescriptorFilterService) comp.Site.GetService (typeof(ITypeDescriptorFilterService));
 				if (filter != null)
-					cache = filter.FilterAttributes (comp, t);
+					cache = filter.FilterAttributes (comp, attributesTable);
 			}
-			
-			ArrayList atts = new ArrayList ();
-			atts.AddRange (t.Values);
-			AttributeCollection attCol = new AttributeCollection (atts);
+
+			Attribute[] attributes = new Attribute[attributesTable.Values.Count];
+			attributesTable.Values.CopyTo (attributes, 0);
+			AttributeCollection attCol = new AttributeCollection (attributes);
 			if (cache)
 				_attributes = attCol;
 			return attCol;
@@ -795,8 +1060,8 @@ public sealed class TypeDescriptor
 			bool cache = true;
 			PropertyInfo[] props = _component.GetType().GetProperties (BindingFlags.Instance | BindingFlags.Public);
 			Hashtable t = new Hashtable ();
-			foreach (PropertyInfo pr in props)
-				t [pr.Name] = new ReflectionPropertyDescriptor (pr);
+			for (int i = props.Length-1; i >= 0; i--)
+				t [props[i].Name] = new ReflectionPropertyDescriptor (props[i]);
 					
 			if (_component.Site != null) 
 			{
@@ -846,14 +1111,30 @@ public sealed class TypeDescriptor
 		{
 			if (_properties != null)
 				return _properties;
-			
-			PropertyInfo[] props = InfoType.GetProperties (BindingFlags.Instance | BindingFlags.Public);
-			ArrayList descs = new ArrayList (props.Length);
-			for (int n=0; n<props.Length; n++)
-				if (props [n].GetIndexParameters ().Length == 0)
-					descs.Add (new ReflectionPropertyDescriptor (props[n]));
 
-			_properties = new PropertyDescriptorCollection ((PropertyDescriptor[]) descs.ToArray (typeof (PropertyDescriptor)), true);
+			Hashtable propertiesHash = new Hashtable (); // name - null
+			ArrayList propertiesList = new ArrayList (); // propertydescriptors
+			Type currentType = InfoType;
+			// Getting properties type by type, because in the case of a property in the child type, where
+			// the "new" keyword is used and also the return type is changed Type.GetProperties returns 
+			// also the parent property. 
+			// 
+			// Note that we also have to preserve the properties order here.
+			// 
+			while (currentType != null && currentType != typeof (object)) {
+				PropertyInfo[] props = currentType.GetProperties (BindingFlags.Instance | BindingFlags.Public | BindingFlags.DeclaredOnly);
+				foreach (PropertyInfo property in props) {
+					if (property.GetIndexParameters ().Length == 0 &&
+					    property.CanRead &&
+					    !propertiesHash.ContainsKey (property.Name)) {
+						propertiesList.Add (new ReflectionPropertyDescriptor (property));
+						propertiesHash.Add (property.Name, null);
+					}
+				}
+				currentType = currentType.BaseType;
+			}
+
+			_properties = new PropertyDescriptorCollection ((PropertyDescriptor[]) propertiesList.ToArray (typeof (PropertyDescriptor)), true);
 			return _properties;
 		}
 	}
