@@ -27,18 +27,20 @@
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
 
-#if NET_2_0 && SECURITY_DEP
+#if NET_2_0
 
 using System.IO;
 using System.Text;
+#if SECURITY_DEP
 using Mono.Security;
 using Mono.Security.Cryptography;
 using MX = Mono.Security.X509;
+#endif
 
 namespace System.Security.Cryptography.X509Certificates {
 
 	public class X509Certificate2 : X509Certificate {
-
+#if SECURITY_DEP
 		private bool _archived;
 		private X509ExtensionCollection _extensions;
 		private string _name = String.Empty;
@@ -84,9 +86,9 @@ namespace System.Security.Cryptography.X509Certificates {
 			Import (rawData, password, keyStorageFlags);
 		}
 
-		public X509Certificate2 (string fileName) : base (fileName) 
+		public X509Certificate2 (string fileName)
 		{
-			Import (fileName, (string)null, X509KeyStorageFlags.DefaultKeySet);
+			Import (fileName, String.Empty, X509KeyStorageFlags.DefaultKeySet);
 		}
 
 		public X509Certificate2 (string fileName, string password)
@@ -177,7 +179,7 @@ namespace System.Security.Cryptography.X509Certificates {
 			get {
 				if (_cert == null)
 					throw new CryptographicException (empty_error);
-				return _cert.ValidUntil;
+				return _cert.ValidUntil.ToLocalTime ();
 			}
 		}
 
@@ -185,7 +187,7 @@ namespace System.Security.Cryptography.X509Certificates {
 			get {
 				if (_cert == null)
 					throw new CryptographicException (empty_error);
-				return _cert.ValidFrom;
+				return _cert.ValidFrom.ToLocalTime ();
 			}
 		}
 
@@ -221,7 +223,12 @@ namespace System.Security.Cryptography.X509Certificates {
 				if (_cert == null)
 					throw new CryptographicException (empty_error);
 
-				if (value is RSA)
+				// allow NULL so we can "forget" the key associated to the certificate
+				// e.g. in case we want to export it in another format (see bug #396620)
+				if (value == null) {
+					_cert.RSA = null;
+					_cert.DSA = null;
+				} else 	if (value is RSA)
 					_cert.RSA = (RSA) value;
 				else if (value is DSA)
 					_cert.DSA = (DSA) value;
@@ -639,7 +646,14 @@ namespace System.Security.Cryptography.X509Certificates {
 		internal MX.X509Certificate MonoCertificate {
 			get { return _cert; }
 		}
+
+#else
+		// HACK - this ensure the type X509Certificate2 and PrivateKey property exists in the build before
+		// Mono.Security.dll is built. This is required to get working client certificate in SSL/TLS
+		public AsymmetricAlgorithm PrivateKey {
+			get { return null; }
+		}
+#endif
 	}
 }
-
 #endif
