@@ -714,7 +714,7 @@ namespace System {
 
 		object IConvertible.ToType (Type type, IFormatProvider provider)
 		{
-			return System.Convert.ToType (m_value, type, provider);
+			return System.Convert.ToType (m_value, type, provider, false);
 		}
 
 #if ONLY_1_1
