@@ -450,7 +450,7 @@ namespace MonoTests.System.Reflection.Emit
 		}
 
 		[Test]
-		public void TestGetCustomAttributes ()
+		public void TestGetCustomAttributes_Incomplete ()
 		{
 			MethodBuilder mb = genClass.DefineMethod (
 				genMethodName (), 0, typeof (void),
@@ -469,6 +469,29 @@ namespace MonoTests.System.Reflection.Emit
 			}
 		}
 
+		[Test]
+		[Category ("NotWorking")]
+		public void TestGetCustomAttributes_Complete ()
+		{
+			MethodBuilder mb = genClass.DefineMethod (
+				genMethodName (), 0, typeof (void),
+				new Type [1] { typeof (int) });
+			mb.GetILGenerator ().Emit (OpCodes.Ret);
+			genClass.CreateType ();
+
+			try {
+				mb.GetCustomAttributes (true);
+				Assert.Fail ("#1");
+			} catch (NotSupportedException) {
+			}
+
+			try {
+				mb.GetCustomAttributes (null, true);
+				Assert.Fail ("#2");
+			} catch (NotSupportedException) {
+			}
+		}
+
 		[Test]
 		public void TestSetCustomAttribute ()
 		{
@@ -562,6 +585,30 @@ namespace MonoTests.System.Reflection.Emit
 			}
 		}
 
+		[Test]
+		public void SetCustomAttribute_SuppressUnmanagedCodeSecurity ()
+		{
+			string mname = genMethodName ();
+
+			TypeBuilder tb = module.DefineType (genTypeName (), TypeAttributes.Public);
+			MethodBuilder mb = tb.DefineMethod (mname, MethodAttributes.Public,
+				typeof (void), new Type [] { typeof (int), typeof (string) });
+			ConstructorInfo attrCtor = typeof (SuppressUnmanagedCodeSecurityAttribute).
+				GetConstructor (new Type [0]);
+			CustomAttributeBuilder caBuilder = new CustomAttributeBuilder (
+				attrCtor, new object [0]);
+			Assert.IsTrue ((mb.Attributes & MethodAttributes.HasSecurity) == 0, "#1");
+			mb.SetCustomAttribute (caBuilder);
+			//Assert.IsTrue ((mb.Attributes & MethodAttributes.HasSecurity) == 0, "#2");
+			mb.GetILGenerator ().Emit (OpCodes.Ret);
+			Type emittedType = tb.CreateType ();
+			MethodInfo emittedMethod = emittedType.GetMethod (mname);
+			Assert.AreEqual (MethodAttributes.HasSecurity, emittedMethod.Attributes & MethodAttributes.HasSecurity, "#3");
+			//Assert.IsTrue ((mb.Attributes & MethodAttributes.HasSecurity) == 0, "#4");
+			object [] emittedAttrs = emittedMethod.GetCustomAttributes (typeof (SuppressUnmanagedCodeSecurityAttribute), true);
+			Assert.AreEqual (1, emittedAttrs.Length, "#5");
+		}
+
 		[AttributeUsage (AttributeTargets.Parameter)]
 		class PrivateAttribute : Attribute
 		{
@@ -810,6 +857,93 @@ namespace MonoTests.System.Reflection.Emit
 			Assert.IsTrue (attr.BestFitMapping, "#2");
 			Assert.IsTrue (attr.ThrowOnUnmappableChar, "#3");
 		}
+
+		public class GenericFoo <T> {
+			public static T field;
+		}
+
+		[Test]
+		public void ILGen_GenericTypeParameterBuilder ()
+		{
+			TypeBuilder tb = module.DefineType (genTypeName (), TypeAttributes.Public);
+			MethodBuilder mb = tb.DefineMethod ("box_int", 
+												MethodAttributes.Public|MethodAttributes.Static, typeof (object), new Type [] { typeof (int) });
+
+			GenericTypeParameterBuilder[] pars = mb.DefineGenericParameters (new string [] { "foo" });
+
+			ILGenerator ilgen = mb.GetILGenerator ();
+			ilgen.Emit (OpCodes.Ldarg_0);
+			ilgen.Emit (OpCodes.Box, pars [0]);
+			ilgen.Emit (OpCodes.Ret);
+
+			Type t = tb.CreateType ();
+			MethodInfo mi = t.GetMethod ("box_int");
+			MethodInfo mi2 = mi.MakeGenericMethod (new Type [] { typeof (int) });
+			Assert.AreEqual (1, mi2.Invoke (null, new object [] { 1 }));
+		}
+
+		public void ILGen_InstantiatedGenericType ()
+		{
+			TypeBuilder tb = module.DefineType (genTypeName (), TypeAttributes.Public);
+			MethodBuilder mb = tb.DefineMethod ("return_type", 
+												MethodAttributes.Public|MethodAttributes.Static, typeof (object), new Type [] { });
+
+			GenericTypeParameterBuilder[] pars = mb.DefineGenericParameters (new string [] { "foo" });
+
+			ILGenerator ilgen = mb.GetILGenerator ();
+
+			Type genericFoo = typeof (GenericFoo<int>).GetGenericTypeDefinition ().MakeGenericType (new Type [] { pars [0] });
+
+			ilgen.Emit (OpCodes.Ldtoken, genericFoo);
+			ilgen.Emit (OpCodes.Call, typeof (Type).GetMethod ("GetTypeFromHandle"));
+			ilgen.Emit (OpCodes.Ret);
+
+			Type t = tb.CreateType ();
+			MethodInfo mi = t.GetMethod ("box_int");
+			MethodInfo mi2 = mi.MakeGenericMethod (new Type [] { typeof (int) });
+			Assert.AreEqual (typeof (GenericFoo<int>), mi2.Invoke (null, new object [] { 1 }));
+		}
+
+		public void ILGen_InstantiatedTypeBuilder ()
+		{
+			TypeBuilder genericTb = module.DefineType (genTypeName (), TypeAttributes.Public);
+			genericTb.DefineGenericParameters (new string [] { "foo" });
+			Type generatedGenericType = genericTb.CreateType ();
+
+			TypeBuilder tb = module.DefineType (genTypeName (), TypeAttributes.Public);
+			MethodBuilder mb = tb.DefineMethod ("return_type", 
+												MethodAttributes.Public|MethodAttributes.Static, typeof (object), new Type [] { });
+
+			GenericTypeParameterBuilder[] pars = mb.DefineGenericParameters (new string [] { "foo" });
+
+			ILGenerator ilgen = mb.GetILGenerator ();
+
+			ilgen.Emit (OpCodes.Ldtoken, genericTb.MakeGenericType (new Type [] { pars [0] }));
+			ilgen.Emit (OpCodes.Call, typeof (Type).GetMethod ("GetTypeFromHandle"));
+			ilgen.Emit (OpCodes.Ret);
+
+			Type t = tb.CreateType ();
+			MethodInfo mi = t.GetMethod ("return_type");
+			MethodInfo mi2 = mi.MakeGenericMethod (new Type [] { typeof (int) });
+			Assert.AreEqual (generatedGenericType.MakeGenericType (new Type [] { typeof (int) }), mi2.Invoke (null, new object [] { 1 }));
+		}
+
+		[Test]
+		public void Bug354757 ()
+		{
+			TypeBuilder gtb = module.DefineType (genTypeName (), TypeAttributes.Public);
+			gtb.DefineGenericParameters ("T");
+			MethodBuilder mb = gtb.DefineMethod ("foo", MethodAttributes.Public);
+			mb.DefineGenericParameters ("S");
+			Assert.IsTrue (mb.IsGenericMethodDefinition);
+
+			Type gt = gtb.MakeGenericType (typeof (object));
+			MethodInfo m = TypeBuilder.GetMethod (gt, mb);
+			Assert.IsTrue (m.IsGenericMethodDefinition);
+
+			MethodInfo mopen = m.MakeGenericMethod (m.GetGenericArguments ());
+			Assert.IsFalse (mopen.IsGenericMethodDefinition);
+		}
 #endif
 	}
 }
