@@ -394,6 +394,43 @@ utf16_concat (const gunichar2 *first, ...)
 	return ret;
 }
 
+#ifdef PLATFORM_MACOSX
+#include <sys/utsname.h>
+
+/* 0 = no detection; -1 = not 10.5 or higher;  1 = 10.5 or higher */
+static int osx_10_5_or_higher;
+
+static void
+detect_osx_10_5_or_higher ()
+{
+	struct utsname u;
+	char *p;
+	int v;
+	
+	if (uname (&u) != 0){
+		osx_10_5_or_higher = 1;
+		return;
+	}
+
+	p = u.release;
+	v = atoi (p);
+	
+	if (v < 9)
+		osx_10_5_or_higher = -1;
+	else 
+		osx_10_5_or_higher = 1;
+}
+
+static gboolean
+is_macos_10_5_or_higher ()
+{
+	if (osx_10_5_or_higher == 0)
+		detect_osx_10_5_or_higher ();
+	
+	return (osx_10_5_or_higher == 1);
+}
+#endif
+
 static const gunichar2 utf16_space_bytes [2] = { 0x20, 0 };
 static const gunichar2 *utf16_space = utf16_space_bytes; 
 static const gunichar2 utf16_quote_bytes [2] = { 0x22, 0 };
@@ -442,7 +479,10 @@ gboolean ShellExecuteEx (WapiShellExecuteInfo *sei)
 			return FALSE;
 
 #ifdef PLATFORM_MACOSX
-		handler = "/usr/bin/open";
+		if (is_macos_10_5_or_higher ())
+			handler = g_strdup ("/usr/bin/open -W");
+		else
+			handler = g_strdup ("/usr/bin/open");
 #else
 		/*
 		 * On Linux, try: xdg-open, the FreeDesktop standard way of doing it,
@@ -1681,6 +1721,48 @@ static gint find_procmodule (gconstpointer a, gconstpointer b)
 	}
 }
 
+#ifdef PLATFORM_MACOSX
+#include <mach-o/dyld.h>
+
+static GSList *load_modules ()
+{
+	GSList *ret = NULL;
+	WapiProcModule *mod;
+	uint32_t count = _dyld_image_count ();
+	int i = 0;
+
+	for (i = 0; i < count; i++) {
+		const struct mach_header *hdr;
+		const struct section *sec;
+		const char *name;
+		intptr_t slide;
+
+		slide = _dyld_get_image_vmaddr_slide (i);
+		name = _dyld_get_image_name (i);
+		hdr = _dyld_get_image_header (i);
+		sec = getsectbynamefromheader (hdr, SEG_DATA, SECT_DATA);
+
+		mod = g_new0 (WapiProcModule, 1);
+		mod->address_start = sec->addr;
+		mod->address_end = sec->addr+sec->size;
+		mod->perms = g_strdup ("r--p");
+		mod->address_offset = 0;
+		mod->device = makedev (0, 0);
+		mod->inode = (ino_t) i;
+		mod->filename = g_strdup (name); 
+		
+		if (g_slist_find_custom (ret, mod, find_procmodule) == NULL) {
+			ret = g_slist_prepend (ret, mod);
+		} else {
+			free_procmodule (mod);
+		}
+	}
+
+	ret = g_slist_reverse (ret);
+	
+	return(ret);
+}
+#else
 static GSList *load_modules (FILE *fp)
 {
 	GSList *ret = NULL;
@@ -1799,6 +1881,7 @@ static GSList *load_modules (FILE *fp)
 	
 	return(ret);
 }
+#endif
 
 static gboolean match_procname_to_modulename (gchar *procname, gchar *modulename)
 {
@@ -1825,7 +1908,7 @@ gboolean EnumProcessModules (gpointer process, gpointer *modules,
 {
 	struct _WapiHandle_process *process_handle;
 	gboolean ok;
-	gchar *filename;
+	gchar *filename = NULL;
 	FILE *fp;
 	GSList *mods = NULL;
 	WapiProcModule *module;
@@ -1863,6 +1946,10 @@ gboolean EnumProcessModules (gpointer process, gpointer *modules,
 		proc_name = process_handle->proc_name;
 	}
 	
+#ifdef PLATFORM_MACOSX
+	{
+		mods = load_modules ();
+#else
 	filename = g_strdup_printf ("/proc/%d/maps", pid);
 	if ((fp = fopen (filename, "r")) == NULL) {
 		/* No /proc/<pid>/maps so just return the main module
@@ -1873,6 +1960,7 @@ gboolean EnumProcessModules (gpointer process, gpointer *modules,
 	} else {
 		mods = load_modules (fp);
 		fclose (fp);
+#endif
 		count = g_slist_length (mods);
 		
 		/* count + 1 to leave slot 0 for the main module */
@@ -1909,7 +1997,7 @@ gboolean EnumProcessModules (gpointer process, gpointer *modules,
 
 static gchar *get_process_name_from_proc (pid_t pid)
 {
-	gchar *filename;
+	gchar *filename = NULL;
 	gchar *ret = NULL;
 	gchar buf[256];
 	FILE *fp;
@@ -1978,7 +2066,7 @@ static guint32 get_module_name (gpointer process, gpointer module,
 	gchar *procname_ext = NULL;
 	glong len;
 	gsize bytes;
-	gchar *filename;
+	gchar *filename = NULL;
 	FILE *fp;
 	GSList *mods = NULL;
 	WapiProcModule *found_module;
@@ -1993,6 +2081,8 @@ static guint32 get_module_name (gpointer process, gpointer module,
 		   __func__, process, module);
 #endif
 
+	size = size*sizeof(gunichar2); /* adjust for unicode characters */
+
 	if (basename == NULL || size == 0) {
 		return(0);
 	}
@@ -2017,6 +2107,10 @@ static guint32 get_module_name (gpointer process, gpointer module,
 	}
 
 	/* Look up the address in /proc/<pid>/maps */
+#ifdef PLATFORM_MACOSX
+	{
+		mods = load_modules ();
+#else
 	filename = g_strdup_printf ("/proc/%d/maps", pid);
 	if ((fp = fopen (filename, "r")) == NULL) {
 		if (errno == EACCES && module == NULL && base == TRUE) {
@@ -2032,6 +2126,7 @@ static guint32 get_module_name (gpointer process, gpointer module,
 	} else {
 		mods = load_modules (fp);
 		fclose (fp);
+#endif
 		count = g_slist_length (mods);
 
 		/* If module != NULL compare the address.
@@ -2127,7 +2222,7 @@ gboolean GetModuleInformation (gpointer process, gpointer module,
 	struct _WapiHandle_process *process_handle;
 	gboolean ok;
 	pid_t pid;
-	gchar *filename;
+	gchar *filename = NULL;
 	FILE *fp;
 	GSList *mods = NULL;
 	WapiProcModule *found_module;
@@ -2166,6 +2261,10 @@ gboolean GetModuleInformation (gpointer process, gpointer module,
 		proc_name = g_strdup (process_handle->proc_name);
 	}
 
+#ifdef PLATFORM_MACOSX
+	{
+		mods = load_modules ();
+#else
 	/* Look up the address in /proc/<pid>/maps */
 	filename = g_strdup_printf ("/proc/%d/maps", pid);
 	if ((fp = fopen (filename, "r")) == NULL) {
@@ -2178,6 +2277,7 @@ gboolean GetModuleInformation (gpointer process, gpointer module,
 	} else {
 		mods = load_modules (fp);
 		fclose (fp);
+#endif
 		count = g_slist_length (mods);
 
 		/* If module != NULL compare the address.
