@@ -15,6 +15,7 @@
 #include "mono/metadata/loader.h"
 #include "mono/metadata/mono-config.h"
 #include "mono/metadata/metadata-internals.h"
+#include "mono/metadata/object-internals.h"
 #include "mono/utils/mono-logger.h"
 
 #if defined(__linux__)
@@ -42,26 +43,40 @@
 
 #if defined(__i386__)
 #define CONFIG_CPU "x86"
+#define CONFIG_WORDSIZE "32"
 #elif defined(__x86_64__)
 #define CONFIG_CPU "x86-64"
+#define CONFIG_WORDSIZE "64"
 #elif defined(sparc) || defined(__sparc__)
 #define CONFIG_CPU "sparc"
+#define CONFIG_WORDSIZE "32"
+#elif defined(__ppc64__) || defined(__powerpc64__)
+#define CONFIG_CPU "ppc64"
+#define CONFIG_WORDSIZE "64"
 #elif defined(__ppc__) || defined(__powerpc__)
 #define CONFIG_CPU "ppc"
+#define CONFIG_WORDSIZE "32"
 #elif defined(__s390x__)
 #define CONFIG_CPU "s390x"
+#define CONFIG_WORDSIZE "64"
 #elif defined(__s390__)
 #define CONFIG_CPU "s390"
+#define CONFIG_WORDSIZE "32"
 #elif defined(__arm__)
 #define CONFIG_CPU "arm"
+#define CONFIG_WORDSIZE "32"
 #elif defined(__ia64__)
 #define CONFIG_CPU "ia64"
+#define CONFIG_WORDSIZE "64"
 #elif defined(__alpha__)
 #define CONFIG_CPU "alpha"
+#define CONFIG_WORDSIZE "64"
 #elif defined(hppa) || defined(__hppa__)
 #define CONFIG_CPU "hppa"
+#define CONFIG_WORDSIZE "32"
 #elif defined(mips) || defined(__mips) || defined(_mips)
 #define CONFIG_CPU "mips"
+#define CONFIG_WORDSIZE "32"
 #else
 #warning Unknown CPU
 #define CONFIG_CPU "unknownCPU"
@@ -244,6 +259,8 @@ dllmap_start (gpointer user_data,
 				info->ignore = TRUE;
 			else if (strcmp (attribute_names [i], "cpu") == 0 && !arch_matches (CONFIG_CPU, attribute_values [i]))
 				info->ignore = TRUE;
+			else if (strcmp (attribute_names [i], "wordsize") == 0 && !arch_matches (CONFIG_WORDSIZE, attribute_values [i]))
+				info->ignore = TRUE;
 		}
 		if (!info->ignore)
 			mono_dllmap_insert (info->assembly, info->dll, NULL, info->target, NULL);
@@ -261,6 +278,8 @@ dllmap_start (gpointer user_data,
 				ignore = TRUE;
 			else if (strcmp (attribute_names [i], "cpu") == 0 && !arch_matches (CONFIG_CPU, attribute_values [i]))
 				ignore = TRUE;
+			else if (strcmp (attribute_names [i], "wordsize") == 0 && !arch_matches (CONFIG_WORDSIZE, attribute_values [i]))
+				ignore = TRUE;
 		}
 		if (!dll)
 			dll = info->dll;
@@ -289,6 +308,31 @@ dllmap_handler = {
 	dllmap_finish
 };
 
+static void
+legacyUEP_start (gpointer user_data, 
+              const gchar         *element_name,
+              const gchar        **attribute_names,
+              const gchar        **attribute_values) {
+	if ((strcmp (element_name, "legacyUnhandledExceptionPolicy") == 0) &&
+			(attribute_names [0] != NULL) &&
+			(strcmp (attribute_names [0], "enabled") == 0)) {
+		if ((strcmp (attribute_values [0], "1") == 0) ||
+				(g_strcasecmp (attribute_values [0], "true") == 0)) {
+			mono_runtime_unhandled_exception_policy_set (MONO_UNHANDLED_POLICY_LEGACY);
+		}
+	}
+}
+
+static const MonoParseHandler
+legacyUEP_handler = {
+	"legacyUnhandledExceptionPolicy",
+	NULL, /* init */
+	legacyUEP_start,
+	NULL, /* text */
+	NULL, /* end */
+	NULL, /* finish */
+};
+
 static int inited = 0;
 
 static void
@@ -297,6 +341,7 @@ mono_config_init (void)
 	inited = 1;
 	config_handlers = g_hash_table_new (g_str_hash, g_str_equal);
 	g_hash_table_insert (config_handlers, (gpointer) dllmap_handler.element_name, (gpointer) &dllmap_handler);
+	g_hash_table_insert (config_handlers, (gpointer) legacyUEP_handler.element_name, (gpointer) &legacyUEP_handler);
 }
 
 /* FIXME: error handling */
